@@ -160,6 +160,10 @@ impl<T: SetAttributes + Draw> ElementInner<T> {
         self.class.as_deref()
     }
 
+    fn inherit_xml_lang(&mut self) {
+        self.specified_values.inherit_xml_lang(&mut self.values);
+    }
+
     fn get_specified_values(&self) -> &SpecifiedValues {
         &self.specified_values
     }
@@ -497,6 +501,10 @@ impl Element {
         call_inner!(self, get_class)
     }
 
+    pub fn inherit_xml_lang(&mut self) {
+        call_inner!(self, inherit_xml_lang)
+    }
+
     pub fn get_specified_values(&self) -> &SpecifiedValues {
         call_inner!(self, get_specified_values)
     }
