@@ -447,7 +447,6 @@ make_properties! {
         "marker-mid"                  => (PresentationAttr::Yes, marker_mid                  : MarkerMid),
         "marker-start"                => (PresentationAttr::Yes, marker_start                : MarkerStart),
         "mask"                        => (PresentationAttr::Yes, mask                        : Mask),
-        // "mask-type"                => (PresentationAttr::Yes, unimplemented),
         "opacity"                     => (PresentationAttr::Yes, opacity                     : Opacity),
         "overflow"                    => (PresentationAttr::Yes, overflow                    : Overflow),
         // "pointer-events"           => (PresentationAttr::Yes, unimplemented),
@@ -485,6 +484,7 @@ make_properties! {
 
     longhands_not_supported_by_markup5ever: {
         "line-height"                 => (PresentationAttr::No,  line_height                 : LineHeight),
+        "mask-type"                   => (PresentationAttr::Yes, mask_type                   : MaskType),
         "mix-blend-mode"              => (PresentationAttr::No,  mix_blend_mode              : MixBlendMode),
         "paint-order"                 => (PresentationAttr::Yes, paint_order                 : PaintOrder),
     }
@@ -542,6 +542,12 @@ impl SpecifiedValues {
             ParsedProperty::Marker(SpecifiedValue::Specified(ref m)) => {
                 self.expand_marker_shorthand(m, replace)
             }
+            ParsedProperty::Font(SpecifiedValue::Inherit) => {
+                self.expand_font_shorthand_inherit(replace)
+            }
+            ParsedProperty::Marker(SpecifiedValue::Inherit) => {
+                self.expand_marker_shorthand_inherit(replace)
+            }
 
             _ => self.set_property(prop, replace),
         }
@@ -605,6 +611,40 @@ impl SpecifiedValues {
         );
     }
 
+    fn expand_font_shorthand_inherit(&mut self, replace: bool) {
+        self.set_property(&ParsedProperty::FontStyle(SpecifiedValue::Inherit), replace);
+        self.set_property(
+            &ParsedProperty::FontVariant(SpecifiedValue::Inherit),
+            replace,
+        );
+        self.set_property(
+            &ParsedProperty::FontWeight(SpecifiedValue::Inherit),
+            replace,
+        );
+        self.set_property(
+            &ParsedProperty::FontStretch(SpecifiedValue::Inherit),
+            replace,
+        );
+        self.set_property(&ParsedProperty::FontSize(SpecifiedValue::Inherit), replace);
+        self.set_property(
+            &ParsedProperty::LineHeight(SpecifiedValue::Inherit),
+            replace,
+        );
+        self.set_property(
+            &ParsedProperty::FontFamily(SpecifiedValue::Inherit),
+            replace,
+        );
+    }
+
+    fn expand_marker_shorthand_inherit(&mut self, replace: bool) {
+        self.set_property(
+            &ParsedProperty::MarkerStart(SpecifiedValue::Inherit),
+            replace,
+        );
+        self.set_property(&ParsedProperty::MarkerMid(SpecifiedValue::Inherit), replace);
+        self.set_property(&ParsedProperty::MarkerEnd(SpecifiedValue::Inherit), replace);
+    }
+
     pub fn set_parsed_property(&mut self, prop: &ParsedProperty) {
         self.set_property_expanding_shorthands(prop, true);
     }
@@ -666,6 +706,7 @@ impl SpecifiedValues {
         compute!(MarkerMid, marker_mid);
         compute!(MarkerStart, marker_start);
         compute!(Mask, mask);
+        compute!(MaskType, mask_type);
         compute!(MixBlendMode, mix_blend_mode);
         compute!(Opacity, opacity);
         compute!(Overflow, overflow);
@@ -688,8 +729,8 @@ impl SpecifiedValues {
         compute!(UnicodeBidi, unicode_bidi);
         compute!(Visibility, visibility);
         compute!(WritingMode, writing_mode);
-        compute!(XmlLang, xml_lang);
         compute!(XmlSpace, xml_space);
+        compute!(XmlLang, xml_lang);
 
         computed.transform = self.transform.unwrap_or_else(|| {
             match self.get_property(PropertyId::TransformProperty) {
@@ -701,6 +742,21 @@ impl SpecifiedValues {
         });
     }
 
+    /// This is a somewhat egregious hack to allow xml:lang to be stored as a presentational
+    /// attribute. Presentational attributes can often be influenced by stylesheets,
+    /// so they're cascaded after selector matching is done, but xml:lang can be queried by
+    /// CSS selectors, so they need to be cascaded *first*.
+    pub fn inherit_xml_lang(&self, computed: &mut ComputedValues) {
+        let prop_val = self.get_property(PropertyId::XmlLang);
+        if let ParsedProperty::XmlLang(s) = prop_val {
+            computed.set_value(ComputedValue::XmlLang(
+                s.compute(&computed.xml_lang(), computed),
+            ));
+        } else {
+            unreachable!();
+        }
+    }
+
     pub fn is_overflow(&self) -> bool {
         if let Some(overflow_index) = self.property_index(PropertyId::Overflow) {
             match self.props[overflow_index] {
