@@ -7,7 +7,7 @@ use crate::bbox::BoundingBox;
 use crate::coord_units::CoordUnits;
 use crate::document::{AcquiredNodes, NodeId};
 use crate::drawing_ctx::{ClipMode, DrawingCtx, ViewParams};
-use crate::element::{Draw, ElementResult, SetAttributes};
+use crate::element::{Draw, Element, ElementResult, SetAttributes};
 use crate::error::*;
 use crate::href::{is_href, set_href};
 use crate::layout::StackingContext;
@@ -208,7 +208,7 @@ impl Svg {
 
         let is_measuring_toplevel_svg = !has_parent && draw_ctx.is_measuring();
 
-        let (viewport, vbox) = if is_measuring_toplevel_svg || has_parent {
+        let (viewport, vbox) = if is_measuring_toplevel_svg {
             // We are obtaining the toplevel SVG's geometry.  This means, don't care about the
             // DrawingCtx's viewport, just use the SVG's intrinsic dimensions and see how far
             // it wants to extend.
@@ -216,7 +216,11 @@ impl Svg {
         } else {
             (
                 // The client's viewport overrides the toplevel's x/y/w/h viewport
-                draw_ctx.toplevel_viewport(),
+                if has_parent {
+                    svg_viewport
+                } else {
+                    draw_ctx.toplevel_viewport()
+                },
                 // Use our viewBox if available, or try to derive one from
                 // the intrinsic dimensions.
                 self.vbox.or_else(|| {
@@ -511,7 +515,7 @@ impl Draw for Mask {}
 
 #[derive(Default)]
 pub struct Link {
-    link: Option<String>,
+    pub link: Option<String>,
 }
 
 impl SetAttributes for Link {
@@ -536,6 +540,14 @@ impl Draw for Link {
         draw_ctx: &mut DrawingCtx,
         clipping: bool,
     ) -> Result<BoundingBox, RenderingError> {
+        // If this element is inside of <text>, do not draw it.
+        // The <text> takes care of it.
+        for an in node.ancestors() {
+            if matches!(&*an.borrow_element(), Element::Text(_)) {
+                return Ok(draw_ctx.empty_bbox());
+            }
+        }
+
         let cascaded = CascadedValues::new(cascaded, node);
         let values = cascaded.get();
 
