@@ -38,6 +38,7 @@ struct Chunk {
     x: Option<f64>,
     y: Option<f64>,
     spans: Vec<Span>,
+    link: Option<String>,
 }
 
 struct MeasuredChunk {
@@ -46,12 +47,14 @@ struct MeasuredChunk {
     y: Option<f64>,
     advance: (f64, f64),
     spans: Vec<MeasuredSpan>,
+    link: Option<String>,
 }
 
 struct PositionedChunk {
     next_chunk_x: f64,
     next_chunk_y: f64,
     spans: Vec<PositionedSpan>,
+    link: Option<String>,
 }
 
 struct Span {
@@ -78,11 +81,12 @@ struct PositionedSpan {
 }
 
 impl Chunk {
-    fn new(values: &ComputedValues, x: Option<f64>, y: Option<f64>) -> Chunk {
+    fn new(values: &ComputedValues, x: Option<f64>, y: Option<f64>, link: Option<String>) -> Chunk {
         Chunk {
             values: Rc::new(values.clone()),
             x,
             y,
+            link,
             spans: Vec::new(),
         }
     }
@@ -110,6 +114,7 @@ impl MeasuredChunk {
             y: chunk.y,
             advance,
             spans: measured_spans,
+            link: chunk.link.clone(),
         }
     }
 }
@@ -183,6 +188,7 @@ impl PositionedChunk {
             next_chunk_x: x,
             next_chunk_y: y,
             spans: positioned,
+            link: measured.link.clone(),
         }
     }
 }
@@ -330,6 +336,7 @@ fn children_to_chunks(
     dx: f64,
     dy: f64,
     depth: usize,
+    link: Option<String>,
 ) {
     for child in node.children() {
         if child.is_chars() {
@@ -352,6 +359,25 @@ fn children_to_chunks(
                         dx,
                         dy,
                         depth + 1,
+                        link.clone(),
+                    );
+                }
+
+                Element::Link(ref link) => {
+                    // TSpan::default tes all offsets to 0,
+                    // which is what we want in links.
+                    let tspan = TSpan::default();
+                    let cascaded = CascadedValues::new(cascaded, &child);
+                    tspan.to_chunks(
+                        &child,
+                        acquired_nodes,
+                        &cascaded,
+                        draw_ctx,
+                        chunks,
+                        dx,
+                        dy,
+                        depth + 1,
+                        link.link.clone(),
                     );
                 }
 
@@ -498,7 +524,7 @@ impl Text {
         let view_params = draw_ctx.get_view_params();
         let params = NormalizeParams::new(values, &view_params);
 
-        chunks.push(Chunk::new(values, Some(x), Some(y)));
+        chunks.push(Chunk::new(values, Some(x), Some(y), None));
 
         let dx = self.dx.to_user(&params);
         let dy = self.dy.to_user(&params);
@@ -512,6 +538,7 @@ impl Text {
             dx,
             dy,
             0,
+            None,
         );
         chunks
     }
@@ -592,7 +619,12 @@ impl Draw for Text {
 
                 for chunk in &positioned_chunks {
                     for span in &chunk.spans {
-                        let span_bbox = span.draw(an, dc, clipping)?;
+                        let span_bbox = match chunk.link.as_ref() {
+                            Some(l) if !l.is_empty() => {
+                                dc.with_link_tag(l.as_str(), &mut |dc| span.draw(an, dc, clipping))?
+                            }
+                            _ => span.draw(an, dc, clipping)?,
+                        };
                         bbox.insert(&span_bbox);
                     }
                 }
@@ -695,6 +727,7 @@ impl TSpan {
         dx: f64,
         dy: f64,
         depth: usize,
+        link: Option<String>,
     ) {
         let values = cascaded.get();
         if !values.is_displayed() {
@@ -711,7 +744,7 @@ impl TSpan {
         let span_dy = dy + self.dy.to_user(&params);
 
         if x.is_some() || y.is_some() {
-            chunks.push(Chunk::new(values, x, y));
+            chunks.push(Chunk::new(values, x, y, link.clone()));
         }
 
         children_to_chunks(
@@ -723,6 +756,7 @@ impl TSpan {
             span_dx,
             span_dy,
             depth,
+            link,
         );
     }
 }
@@ -835,7 +869,7 @@ fn create_pango_layout(draw_ctx: &DrawingCtx, props: &FontProperties, text: &str
     let pango_context = draw_ctx.create_pango_context();
 
     if let XmlLang(Some(ref lang)) = props.xml_lang {
-        pango_context.set_language(&pango::Language::from_string(lang));
+        pango_context.set_language(&pango::Language::from_string(lang.as_str()));
     }
 
     pango_context.set_base_gravity(pango::Gravity::from(props.writing_mode));
